'use strict'
const gulp = require('gulp');
const htmlparser = require("htmlparser2");
const fs = require("fs");
const through2 = require("through2");

class HTMLAudit {
    constructor() {
        this.auditResults = [];
    }

    parse(file, encoding, callback) {
        const parser = new htmlparser.parseDOM(String(file.contents));

        this.currentAuditFile = {
            file: file.path,
            numberOfErrors: 0,
            errors: [],

            addError(error) {
                this.errors.push(error);
                this.numberOfErrors = this.errors.length;
            }
        };

        this.auditResults.push(this.currentAuditFile);

        for(let element of parser) {
            this.audit(element);
        }

        callback();
    }

    audit(element) {
        if (element.name === "ul") {
            this.validateItHasRole(element, "ul");
        }
        else if (element.name === "ol") {
            this.validateItHasRole(element, "ol")
        }
        else if (element.name === "li") {
            this.validateItHasRole(element, "li")
        }
        else if (element.name === "img") {
            this.validateImage(element);
        }
        else if (element.name === "button") {
            this.validateButton(element);
        }
        else if (element.name == "pragma-input") {
            this.validatePragmaInput(element);
        }
        else if (element.name == "input") {
            this.validateInput(element);
        }

        if (element.children && element.children.length > 0) {
            for(let childElement of element.children) {
                this.audit(childElement);
            }
        }
    }

    printResults() {
        let totalErrors = 0;
        for(let audit of this.auditResults) {
            totalErrors += audit.numberOfErrors;
        }


        if (totalErrors === 0) {
            console.warn('Audit passed, well done!');
            return;
        }

        var dir = './coverage';

        if (!fs.existsSync(dir)){
            fs.mkdirSync(dir);
        }

        var fileName = dir +  `/html-audit.json`;
        var textToSave = JSON.stringify(this.auditResults, null, 4);

        fs.writeFileSync(fileName, textToSave);
    }

    validateItHasRole(element, type) {
        const role = element.attribs.role;

        if (!role) {
            const reference = element.children && element.children.length > 0 ? element.children["0"].data : null;

            let error = type;

            if (reference && reference.trim().length > 0)
            {
                error += ` with value "${reference}"`;
            }

            error += " must have a role attribute defined";

            this.currentAuditFile.addError(error);
            return;
        }

        this.validateRoleWithParent(element);
    }


    validateRoleWithParent(element) {
        if (!element.parent) {
            return;
        }

        const parentRole = element.parent.attribs.role;
        const childRole = element.attribs ? element.attribs.role : null;

        if (!parentRole || !childRole) {
            // not having a role would have been logged as a different error
            return;
        }

        if (parentRole.toLowerCase() === "menu" && childRole.toLowerCase() !== "menuitem") {
            this.currentAuditFile.addError('children of a menu must have the role of "menuitem"');
        }
    }

    validateImage(element) {
        const alt = element.attribs.alt;
        const src = element.attribs.src;

        if (!alt) {
            this.currentAuditFile.addError(`image with source "${src}" must have alt attribute`)
        }
    }

    validateButton(element) {
        const ariaLabel = element.attribs["aria-label"];

        if (!ariaLabel) {
            this.currentAuditFile.addError('button must have aria-label attribute');
        }
    }

    validatePragmaInput(element) {
        const id = element.attribs.id;

        if (!id) {
            this.currentAuditFile.addError('pragma-input must have a id attribute defined');
        }
    }

    validateInput(element) {
        const describedby = element.attribs["aria-describedby"];
        const label = element.attribs["aria-label"];
        const labelby = element.attribs["aria-labelledby"];
        const type = element.attribs.type;

        if (!describedby) {
            this.currentAuditFile.addError('input must have a aria-describedby attribute');
        }

        if (!type) {
            this.currentAuditFile.addError('input must have a type attribute');
        }

        if (!label && !labelby) {
            this.currentAuditFile.addError('input must have either aria-label or aria-labelledby defined');
        }
    }

    hasAttribute(lineToProcess, attribute) {
        return lineToProcess.text.toLowerCase().indexOf(`${attribute}="`) > -1;
    }
}

function auditAccessability(path) {
    return new Promise((resolve) => {
        const audit = new HTMLAudit();

        return gulp.src(path ? path : 'src/**/*.html')
            .pipe(through2({objectMode: true}, audit.parse.bind(audit)))
            .on('finish', function() {
                audit.printResults();
                resolve();
            });
    });
}

gulp.task('audit:html', function() {
    return auditAccessability();
});

module.exports = auditAccessability;