"use strict";
const gulp = require("gulp");
const through2 = require('through2');
const fs = require("fs");

let ignoredCoverageFiles = [];

class AuditFile {
    constructor(fileName) {
        this.fileName = fileName;
        this.classes = [];
    }
}

class Audit {
    constructor() {
        this.audit = [];
        this.currentAudit = null;
    }

    process(path) {
        return new Promise(function(resolve) {
            gulp.src(path)
                .pipe(through2({objectMode: true}, this.processFile.bind(this)))
                .on("finish", resolve);
        }.bind(this))
    }

    processFile(file, encoding, callback) {
        const auditFile = new AuditFile(file.path);
        this.currentAudit = auditFile;
        this.audit.push(auditFile);

        const contents = String(file.contents).split('\n');

        for (let line of contents) {
            if (!this.processLine(line))
            {
                break;
            }
        }

        callback();
    }

    processLine(line) {
        // override this, return true if you can continue processing, false if processing needs to stop
        return true;
    }
}

/**
    CodeAudit is used to if @class is present
 */
class CodeAudit extends Audit {
    processLine(line) {
        const classIndex = line.indexOf("class ");

        if (classIndex > 0 && line[classIndex - 1] !== " ") {
            return;
        }

        if (classIndex > -1) {
            const openIndex = line.indexOf("{");
            const extendsIndex = line.indexOf(" extends");
            const implementsIndex = line.indexOf(" implements");

            const endIndex = extendsIndex > -1 ? extendsIndex : implementsIndex > -1 ? implementsIndex : openIndex;

            const className = line.substring(classIndex + 6, endIndex).trim();

            if (this.currentAudit.classes.indexOf(className) === -1) {
                this.currentAudit.classes.push(className);
            }
        }

        return true;
    }
}

class TestsAudit extends Audit {
    processLine(line) {
        if (line.indexOf('describe(') > -1) {
            return false;
        }

        var classDeclarations = line.substring(6, line.indexOf('from')).trim().replace(/[^a-zA-Z0-9,]/g, '');
        var classes = classDeclarations.split(',');
        Array.prototype.push.apply(this.currentAudit.classes, classes);
        return true;
    }
}

function checkForMissingTests(codeAudit, testsAudit) {
    return new Promise(function(resolve) {
        const auditResult = [];

        for(let audit of codeAudit.audit) {
            const missingTests = [];

            for (let className of audit.classes) {
                if (!isInTests(className, testsAudit)) {
                    missingTests.push(className);
                }
            }

            if (missingTests.length > 0) {
                auditResult.push({
                    fileName: audit.fileName,
                    missingTests: missingTests
                });
            }
        }

        if (auditResult.length > 0) {
            saveAuditToFile(auditResult);
        }
        else {
            console.warn("Missing tests audit passed, there are no missing tests");
        }

        resolve();
    });
}

function isInTests(className, testAudit) {
    for (let audit of testAudit.audit) {
        if (audit.classes.indexOf(className) > -1) {
            return true;
        }
    }

    return false;
}

function saveAuditToFile(auditResults) {
    var dir = './coverage';

    if (!fs.existsSync(dir)){
        fs.mkdirSync(dir);
    }

    var fileName = dir +  '/missing-tests-audit.json';
    var textToSave = JSON.stringify(auditResults, null, 4);

    fs.writeFile(fileName, textToSave);

    printResultSummary(fileName, auditResults);
}

function printResultSummary(fileName, auditResults) {
    console.error('******************************************************\n');
    console.error(`Audit (missing tests) failed, please review audit files: ${fileName}\n`);

    var errorCount = 0;
    for(var i = 0; i < auditResults.length; i++) {
        errorCount += auditResults[i].missingTests.length;
    }

    console.error(`${errorCount} errors over ${auditResults.length} files\n`);

    console.error('******************************************************');
}

function auditMissingTests(sourcePath) {
    const codeAudit = new CodeAudit();
    const testsAudit = new TestsAudit();

    let sourceArray = [];
    if (sourcePath) {
        if (Array.isArray(sourcePath)) {
            sourceArray = sourcePath;
        }
        else
        {
            sourceArray.push(sourcePath);
        }
    }

    let filesToCover = sourcePath ? sourceArray : ["src/**/*.js"];
    filesToCover.push.apply(filesToCover, ignoredCoverageFiles);

    const codeAuditPromise = codeAudit.process(filesToCover);
    const testsAuditPromise = testsAudit.process("test/**/*.js");

    return Promise.all([codeAuditPromise, testsAuditPromise])
        .then(() => {
            checkForMissingTests(codeAudit, testsAudit)
        });
}


gulp.task("audit:missing-tests", ["audit-get-ignored"], function() {
    auditMissingTests();
});

function getIgnored(file, encoding, callback) {
    let object = JSON.parse(String(file.contents));

    const buildUtilities = object["build-utilities"];
    if (!buildUtilities) {
        callback();
        return;
    }

    const ignoredSource = buildUtilities["ignore-in-coverage"];
    ignoredCoverageFiles = ignoredSource ? ignoredSource : [];

    callback();
}

gulp.task('audit-get-ignored', function() {
    return gulp.src("package.json")
        .pipe(through2({objectMode: true}, getIgnored))
});

module.exports = auditMissingTests;