var gulp = require('gulp-param')(require('gulp'), process.argv);
var fsPath = require('fs-path');
var run = require('gulp-run');
var fs = require('fs');

var createComponents = module.exports = {
    isTesting: false,

    createFile: function(fileName, content) {
        if(this.isTesting) {
            return;
        }

        return new Promise(function(resolve) {
            fsPath.writeFile(fileName, content, function(error) {
                if (error) {
                    console.error(error);
                }

                resolve();
            });
        })
    },

    constructProperName: function(name) {
        var result = name.replace(/([A-Z]+)/g, "-$1").replace(/^,/, "");

        if (result[0] === "-") {
            result = result.substr(1);
        }

        return result.toLowerCase();
    },

    constructClassName: function(name) {
        return name.charAt(0).toUpperCase() + name.slice(1);
    },

    constructVariableName: function(name) {
        return name.charAt(0).toLowerCase() + name.slice(1);
    },

    constructPathForTest: function(path) {
        var splitPath = path.split('/');
        var splitOffset = splitPath.length - 3;

        var splitIndent = '';
        for (var i = 0; i < splitOffset; i++) {
            splitIndent += '../';
        };

        result = './' + splitIndent + path.slice(2, path.length - 3);

        return result;
    },

    createViewJsFileContent: function(className) {
        return "" +
`
// @flow
export class ${className} {

}
`;
    },

    createViewHtmlFileContent: function() {
      return "<template>\n</template>";
    },

    createComponentJsFileContent: function(className, properName) {
        return "" +  
`
// @flow
import {customElement, useShadowDOM, inject} from 'aurelia-framework';
@customElement('${properName}')
@inject(Element)
export class ${className} {
    element = null;
    
    constructor(element) {
        this.element = element;
    }
}
`
    },

    createComponentHtmlFileContent: function(properName) {
        return "" +
`
<template>
    <require from="${properName}.css"></require>
    <div>Control Content</div>
</template>
`;
    },

    createClassSourceContent: function(className) {
        return "" +
`
// @flow
export class ${className} {
    constructor() {
    }
}
`;
    },

    createClassTestContent: function(className, importPath, constructorParams) {
        var classVariaibleName = this.constructVariableName(className);


        return "" +
`
import {expect} from 'chai';
import 'aurelia-polyfills';
import {${className}} from './.${importPath}';

describe('${className} Tests', function() {
    let ${classVariaibleName};

    beforeEach(function() {
        ${classVariaibleName} = new ${className} (${constructorParams});
    });
    
    it('constructor', function() {
        expect(${classVariaibleName}).to.not.be.null;
    });
    
    it('not constructor', function() {
        expect(() => ${className}()).to.throw("Cannot call a class as a function");
    });    
})
`;
    },

    createComponentSCSSFileContent: function(componentName) {
        return componentName + "{\n}";
    },

    createComponent: function (name) {
        if (!name) {
            console.error('please provide a name using "--name componentName"');
            return;
        }

        if (name.indexOf(" ") > -1) {
            console.error('no whitespace is allowed in component names');
            return;
        }


        var properFileName = this.constructProperName(name);
        var className = this.constructClassName(name);

        var componentFolder = "./src/components/" + properFileName;
        var jsFile = componentFolder + "/" + properFileName + ".js";
        var htmlFile = componentFolder + "/" + properFileName + ".html";
        var scssFile = componentFolder + "/" + properFileName + ".scss";
        var testFile = "./test/components/" + properFileName + "/" + properFileName + "-tests.js";

        var jsFileContent = this.createComponentJsFileContent(className, properFileName);
        var htmlFileContent = this.createComponentHtmlFileContent(properFileName);
        var scssFileContent = this.createComponentSCSSFileContent(properFileName);

        var sourcePathForTest = this.constructPathForTest(jsFile);
        var testFileContent = this.createClassTestContent(className, sourcePathForTest, "{}");

        var p1 = this.createFile(jsFile, jsFileContent);
        var p2 = this.createFile(htmlFile, htmlFileContent);
        var p3 = this.createFile(scssFile, scssFileContent);
        var p4 = this.createFile(testFile, testFileContent);

        Promise.all([p1, p2, p3, p4]).then(function() {
            run('git add ' + componentFolder.slice(2)).exec();
            run('git add test/components/' + properFileName + '/*').exec();
        });
    },

    createView: function(name) {
        if (!name) {
            console.error('please provide a name using "--name viewName"');
            return;
        }

        if (name.indexOf(" ") > -1) {
            console.error('no whitespace is allowed in view names');
            return;
        }

        var properFileName = this.constructProperName(name);
        var className = this.constructClassName(name);

        var folder = "./src/views/" + properFileName;
        var jsFile = folder + "/" + properFileName + ".js";
        var htmlFile = folder + "/" + properFileName + ".html";
        var testFile = "./test/views/" + properFileName + "/" + properFileName + "-tests.js";

        var jsFileContent = this.createViewJsFileContent(className);
        var htmlFileContent = this.createViewHtmlFileContent();

        var sourcePathForTest = this.constructPathForTest(jsFile);
        var testFileContent = this.createClassTestContent(className, sourcePathForTest, "");

        var p1 = this.createFile(jsFile, jsFileContent);
        var p2 = this.createFile(htmlFile, htmlFileContent);
        var p3 = this.createFile(testFile, testFileContent)

        Promise.all([p1, p2, p3]).then(function() {
            run('git add ' + folder.slice(2)).exec();
            run('git add test/views/' + properFileName + '/*').exec();
        });
    },

    createClass: function(name, path) {
        if (!name) {
            console.error('please provide a name using "--name viewName"');
            return;
        }

        var properFileName = this.constructProperName(name);
        var className = this.constructClassName(name);

        var sourcePathFolder = './src/';
        var testPathFolder = './test/';

        if (path) {
            sourcePathFolder += path + '/';
            testPathFolder += path + '/';
        }

        var sourcePath = sourcePathFolder + properFileName + '.js';
        var testPath = testPathFolder + properFileName + '-tests.js';

        var sourcePathForTest = this.constructPathForTest(sourcePath);

        var jsSourseContent = this.createClassSourceContent(className);
        var jsTestContent = this.createClassTestContent(className, sourcePathForTest, "");

        var p1 = this.createFile(sourcePath, jsSourseContent);
        var p2 = this.createFile(testPath, jsTestContent);

        Promise.all([p1, p2]).then(function() {
            run('git add ' + sourcePathFolder + '*').exec();
            run('git add ' + testPathFolder + '*').exec();
        })
    }
};

gulp.task("create:component", function(name) {
    createComponents.createComponent(name);
});

gulp.task('create:view', function(name) {
   createComponents.createView(name);
});

gulp.task('create:class', function(name, path){
    createComponents.createClass(name, path ? path : '');
});
