"use strict";

const fs = require('fs');

class Report {
    constructor() {
        this.e2eLinks = [];

        this.templates = {
            missing_tests: "missing_tests",
            html_audit: "html_audit",
            code_coverage: "code_coverage",
            e2e: "e2e",
            tests: "tests"
        };

        this.coverage = {
            missing_tests: null,
            html_audit: null,
            code_coverage: null,
            e2e: null,
            tests: null
        };

        this.checkAvailableCoverageFiles();
    }

    checkAvailableCoverageFiles() {
        this.loadMissingTests();
        this.loadHtmlAudit();
        this.loadCoverage();
        this.loadE2E();
        this.loadTests();
        this.generateReport();
    }

    getMissingTestsErrorCount() {
        let count = 0;

        for(let test of this.coverage.missing_tests) {
            count += test.missingTests.length;
        }

        return count;
    }

    getTestsErrorCount() {
        return this.coverage.tests.numberOfFailedTests + (this.coverage.tests.pending ? this.coverage.tests.pending.length : 0);
    }

    getCoverageErrorCount() {
        const listOfFiles = Object.keys(this.coverage.code_coverage);

        const count = {
            pass: 0,
            fail: 0
        };

        for(let file of listOfFiles) {
            const fileObject = this.coverage.code_coverage[file];
            const fKeys = Object.keys(fileObject.f);

            let fResult = this.countPassFailFromKeys(fKeys, fileObject.f);

            count.pass += fResult.pass;
            count.fail += fResult.fail;
        }

        return Math.round(count.pass * 100 / (count.pass + count.fail));
    }

    countPassFailFromKeys(keys, object) {
        const count = {
            pass: 0,
            fail: 0
        };

        for(let key of keys) {
            if (parseInt(object[key]) > 0) {
                count.pass++
            }
            else {
                count.fail++;
            }
        }

        return count;
    }

    getAuditErrorCount() {
        let count = 0;

        for (let audit of this.coverage.html_audit) {
            count += audit.errors.length;
        }

        return count;
    }

    getEndToEndErrorCount() {
        return this.coverage.e2e.totalErrorCount;
    }

    loadMenu() {
        let importText = "";

        if (this.coverage.missing_tests) {
            const missginTestsCount = this.getMissingTestsErrorCount();
            importText += this.createMenuItem("Missing tests", "missing_tests_container", missginTestsCount, missginTestsCount === 0);
        }

        if (this.coverage.tests) {
            const testCount = this.getTestsErrorCount();
            importText += this.createMenuItem("Test needing attention", "tests_container", testCount, testCount === 0);
        }

        if (this.coverage.code_coverage) {
            const coverageCount = this.getCoverageErrorCount();
            importText += this.createMenuItem(`Code coverage - ${coverageCount}%`, "code_coverage_container", "", coverageCount === 100);
        }

        if (this.coverage.html_audit) {
            const auditCount = this.getAuditErrorCount();
            importText += this.createMenuItem("Html audit", "html_audit_container", auditCount, auditCount === 0);
        }

        if (this.coverage.e2e) {
            const e2eCount = this.getEndToEndErrorCount();
            importText += this.createMenuItem("End to end", "e2e_container", e2eCount, e2eCount === 0);
        }

        if (fs.existsSync(__dirname + "/templates/menu.html")) {
            let text = fs.readFileSync(__dirname + "/templates/menu.html", "utf8");
            return text.replace("${import}", importText);
        }

        return "";
    }

    createMenuItem(name, id, errorCount, success) {
        return `<li data-target="${id}" role="aria-menuitem" onclick="auditController.menuitem_selected(event)">
                    <div class="menu-error" success="${success}">${errorCount}</div>
                    <span>${name}</span>                                       
                </li>`;
    }

    loadMissingTests() {
        if (fs.existsSync("coverage/missing-tests-audit.json")) {
            const text = fs.readFileSync("coverage/missing-tests-audit.json");
            this.coverage.missing_tests = JSON.parse(text);
        }
    }

    loadHtmlAudit() {
        if (fs.existsSync("coverage/html-audit.json")) {
            const text = fs.readFileSync("coverage/html-audit.json");
            this.coverage.html_audit = JSON.parse(text);
        }
    }

    loadCoverage() {
        if (fs.existsSync("coverage/coverage-final.json")) {
            const text = fs.readFileSync("coverage/coverage-final.json");
            this.coverage.code_coverage = JSON.parse(text);
        }
    }

    loadE2E() {
        if (fs.existsSync("coverage/e2e_results/summary.json")) {
            const text = fs.readFileSync("coverage/e2e_results/summary.json");
            this.coverage.e2e = JSON.parse(text);
        }
    }

    loadTests() {
        if (fs.existsSync("coverage/test-results.json")) {
            const text = fs.readFileSync("coverage/test-results.json");
            this.coverage.tests = JSON.parse(text);
        }
    }

    generateReport() {
        this.generateMissingTestsPage();
        this.generateHtmlAuditPage();
        this.generateCoveragePage();
        this.generateE2EPage();
        this.generateTestsPage();
        this.generateIndexPage();
    }

    createImportText(name) {
        return `<link rel="import" href="./${name}.html" id="${name}"> \n`
    }

    generateIndexPage() {
        let importText = "";

        if (this.coverage.missing_tests) {
            importText += this.createImportText(this.templates.missing_tests);
        }

        if (this.coverage.html_audit) {
            importText += this.createImportText(this.templates.html_audit);
        }

        if (this.coverage.code_coverage) {
            importText += this.createImportText(this.templates.code_coverage);
        }

        if (this.coverage.e2e) {
            importText += this.createImportText(this.templates.e2e);

            for (let link of this.e2eLinks) {
                importText += this.createImportText(link.split('.html')[0]);
            }
        }

        if (this.coverage.tests) {
            importText += this.createImportText(this.templates.tests);
        }

        let indexHtml = fs.readFileSync(__dirname +  "/templates/index.html", "utf8");
        indexHtml = indexHtml.replace("${imports}", importText);

        let menuHtml = this.loadMenu();
        indexHtml = indexHtml.replace("${menu}", menuHtml);

        fs.writeFileSync("coverage/index.html", indexHtml);

        let indexJs = fs.readFileSync(__dirname +  "/templates/index.js", "utf8");
        fs.writeFileSync("coverage/index.js", indexJs);
    }

    generateTestsPage() {
        let text = "";

        if (this.coverage.tests) {
            if (this.coverage.tests.numberOfFailedTests > 0) {
                for(let test of this.coverage.tests.failed) {
                    text += `
                            <label>File Name</label>
                            <div class="space-below">${test.file}</div>
                            <label>Test Title</label>
                            <div class="space-below">${test.title}</div>
                            <label>Error Message</label>
                            <div success="false" class="space-below">${test.error}</div>
                            <div success="false" class="space-below">${test.stack}</div>
                            <hr />
                    `;
                }
            }
            
            if (this.coverage.tests.pending && this.coverage.tests.pending.length > 0) {
                text += `
                    <h2>Pending Tests</h2>
                `;
                
                for(let test of this.coverage.tests.pending) {
                    text += `
                        <label>File Name</label>
                        <div class="space-below">${test.file}</div>
                        <label>Test Title</label>
                        <div class="space-below">${test.title}</div>
                    `
                }
            }
        }

        let indexHtml = fs.readFileSync(__dirname + `/templates/tests.html`, "utf8");
        indexHtml = indexHtml.replace("${import}", text);
        fs.writeFileSync("coverage/tests.html", indexHtml);
    }

    generateMissingTestsPage() {
        let text = "";

        if (this.coverage.missing_tests) {
            for (let missingTest of this.coverage.missing_tests) {
                text += `
                <label>File Name</label>
                <div class="space-below">${missingTest.fileName}</div>
                <label>Missing Tests</label>
                <ol>
            `;

                for (let className of missingTest.missingTests) {
                    text += `<li>${className}</li>`;
                }

                text += "</ol><hr />";
            }
        }

        let indexHtml = fs.readFileSync(__dirname + `/templates/missing_tests.html`, "utf8");
        indexHtml = indexHtml.replace("${import}", text);
        fs.writeFileSync("coverage/missing_tests.html", indexHtml);
    }

    generateHtmlAuditPage() {
        let text = "";

        if (this.coverage.html_audit) {
            for (let htmlAudit of this.coverage.html_audit) {
                if (htmlAudit.numberOfErrors > 0) {
                    text += `
                <label>File Name</label>
                <div class="space-below">${htmlAudit.file}</div>
                <label>Errors in HTML</label>
                <ol>
            `;

                    for (let error of htmlAudit.errors) {
                        text += `<li>${error}</li>`;
                    }

                    text += "</ol><hr />";
                }
            }
        }
        
        let indexHtml = fs.readFileSync(__dirname + `/templates/html_audit.html`, "utf8");
        indexHtml = indexHtml.replace("${import}", text);
        fs.writeFileSync("coverage/html_audit.html", indexHtml);
    }

    generateCoveragePage() {
        let indexHtml = fs.readFileSync(__dirname + `/templates/code_coverage.html`, "utf8");
        fs.writeFileSync("coverage/code_coverage.html", indexHtml);
    }

    generateE2EPage() {
        let text = "";

        if (this.coverage.e2e && this.coverage.e2e.failedTests) {
            for (let failedTest of this.coverage.e2e.failedTests) {
                const htmlLink = this.generateE2EDetailsPage(failedTest);

                text += `
                <label>Test Source</label>
                <div class="space-below">${failedTest.testSource}</div>
                
                <label>Test Result</label>
                <div href="${htmlLink}" class="space-below" onclick="auditController.showDetail(event)">${failedTest.testResult}</div>

                <label>Number Of Errors</label>
                <div success="${failedTest.errorCount <= 0}">${failedTest.errorCount}</div>
                <hr />
            `;
            }
        }

        if (text.length === 0) {
            text = "All tests passed"
        }

        let indexHtml = fs.readFileSync(__dirname + `/templates/e2e.html`, "utf8");
        indexHtml = indexHtml.replace("${import}", text);
        fs.writeFileSync("coverage/e2e.html", indexHtml);
    }

    generateE2EDetailsPage(failedTest) {
        let indexHtml = fs.readFileSync(__dirname + `/templates/e2e-details.html`, "utf8");
        let content = fs.readFileSync(failedTest.testResult, "utf8");
        const json = JSON.parse(content);

        let text = "";
        for (let step of json.steps) {
            text += `
                <div>
                    <span success="${step.success == true}">${step.success ? "success" : "failed"}</span>
                    <code success="${step.success == true}">${JSON.stringify(step.details, null, 4)}</code>
                </div>
            `
        }
        
        const fileName = this.calculateE2EHtmlPageName(failedTest.testResult);

        indexHtml = indexHtml.replace("${name}", failedTest.testSource).replace("${import}", text);
        fs.writeFileSync(`coverage/${fileName}`, indexHtml);

        this.e2eLinks.push(fileName);

        return fileName;
    }

    calculateE2EHtmlPageName(fileName) {
        let result = fileName.slice(2);
        result = result.split("/").join("-");
        result = result.replace(".json", ".html");

        return result;
    }
}

module.exports = Report;