"use strict";
const gulp = require('gulp');
const mocha = require('gulp-mocha');
const paths = require('./paths');
const istanbul = require('gulp-istanbul');
const through2 = require('through2');
const AutomationTestRunner = require('./automation-test-runner');
const Report = require('./report/report');
const jsonTestsToRun = [];
const browserSync = require('browser-sync');
const Github = require('./github');
const auditMissingTests = require("./audit-missing-tests");
const auditAccessability = require("./audit-accessability");
const testReporter = require('./mocha-test-reporter');
const plumber = require('gulp-plumber');

let ignoredCoverageFiles = [];

function testUnit(path) {
    global.Element = {};

    return new Promise((resolve) => {
        return gulp.src(path ? path : paths.compiledTestFiles)
            .pipe(plumber())
            .pipe(mocha({
                reporter: testReporter
            }))
            .on("end", () => {
                console.log("--- testing done ---");
                resolve();
            })
    });
}

function getAppTestsFromCommit(commit) {
    const tests = commit.tests.slice(0);

    for (var i = 0; i < tests.length; i++) {
        tests[i] = "app/" + tests[i]; //.replace(".ts", ".js");
    }

    return tests;
}

function getAppSourceFromCommit(commit) {
    const source = commit.source.slice(0);

    for (var i = 0; i < source.length; i++) {
        source[i] = "app/" + source[i]; //.replace(".ts", ".js");
    }

    return source;
}

function testUnitFromCommit(commit) {
    const tests = getAppTestsFromCommit(commit);
    return testUnit(tests);
}

function coverageUnit(sourcePath, testPath) {
    global.Element = {};

    let sourceArray = [];
    if (sourcePath) {
        if (Array.isArray(sourcePath)) {
            sourceArray = sourcePath;
        }
        else
        {
            sourceArray.push(sourcePath);
        }
    }

    let filesToCover = sourcePath ? sourceArray : paths.compiledSourceFiles;
    filesToCover.push.apply(filesToCover, ignoredCoverageFiles);

    return gulp.src(filesToCover)
        .pipe(plumber())
        .pipe(istanbul({
            includeUntested: true
        }))
        .pipe(istanbul.hookRequire())
        .on('finish', function() {
            return gulp.src(testPath ? testPath : paths.compiledTestFiles)
                .pipe(plumber())
                .pipe(mocha({
                    reporter: testReporter
                }))
                .pipe(istanbul.writeReports())
        })
}

function testCoverageFromCommit(commit) {
    /*
        You don't know what tests all test different functions for the source that has changed.
        So only check the coverage of the source files, but run all the tests.
     */

    const source = getAppSourceFromCommit(commit);
    return coverageUnit(source, null);
}

function getTestsObjectsFromSource(file, encoding, callback) {
    var test = JSON.parse(String(file.contents));
    test.test.path = file.path;

    jsonTestsToRun.push(test);

    callback();
}

gulp.task('test', ['build:tests'], function(query) {
    return testUnit(query);
});

gulp.task('test:coverage', ['build:tests', 'coverage-get-ignored'], function() {
    return coverageUnit();
});

gulp.task('test:e2e', ['clean:e2e-results'], function(query) {
    const serverOptions = {
        port: 8080,
        open: false,
        server: {
            baseDir: './'
        },
        ui: {
            port: 8081
        }
    };

    const server = browserSync.init(serverOptions, function() {
        var testRunner = new AutomationTestRunner('http://127.0.0.1:8080');

        gulp.src(query ? query : paths.testJsonFiles)
            .pipe(through2({objectMode: true}, getTestsObjectsFromSource))
            .on('finish', function() {
                testRunner.run(jsonTestsToRun, function() {
                    server.exit();
                });
            });
    });
});

gulp.task('test:everything', ['audit:all', 'test:coverage', 'test:e2e']);

gulp.task('test:generate-report', function() {
    const report = new Report();
});

gulp.task('test:commit', ["clean:coverage"], function(sha) {
    const github = new Github();
    github.getFilesOfCommit(sha).then(result => {
        if (result.html.length > 0) {
            auditAccessability(result.html);
        }

        if (result.tests.length > 0) {
            testUnitFromCommit(result);
        }

        if (result.source.length > 0) {
            auditMissingTests(result.source);
            testCoverageFromCommit(result);
        }
    });
});

function getIgnored(file, encoding, callback) {
    let object = JSON.parse(String(file.contents));

    const buildUtilities = object["build-utilities"];
    if (!buildUtilities) {
        callback();
        return;
    }

    const ignoredSource = buildUtilities["ignore-in-coverage"];
    ignoredCoverageFiles = ignoredSource ? ignoredSource : [];

    for (let i = 0; i < ignoredCoverageFiles.length; i++) {
        const ignore = ignoredCoverageFiles[i];
        ignoredCoverageFiles[i] = ignore.replace("!", "!app/");
    }

    callback();
}

gulp.task('coverage-get-ignored', function() {
    return gulp.src("package.json")
        .pipe(through2({objectMode: true}, getIgnored))
});